(******************************************************************************

  Components to Clipboard
  Designed specifically to work with non-visual components
  Jim McKeeth, Embarcadero Technologies
  More information: http://delphi.org/?p=1523
  
(******************************************************************************)   

unit uCopyComponents;

interface

uses
  System.SysUtils, System.Types, System.Classes, Vcl.Clipbrd;

procedure StreamToClipboard(aComponents: Array of TComponent);
function ComponentToText(aComponent: TComponent): string;

implementation

procedure StreamToClipboard(aComponents: Array of TComponent);
var
  componentsText, compText, names: TStringList;
  comp: TComponent;
  lineIdx, nameIdx: Integer;
  line, defName, curName: string;
begin
  names := nil;
  compText := nil;
  componentsText := nil; 
  try
    componentsText := TStringList.Create;
    names := TStringList.Create;
    compText := TStringList.Create;
    for comp in aComponents do
    begin
      if Assigned(Comp) then
      begin

        compText.Text := ComponentToText(comp);
        // Give everything default names
        curName := format('%s.%s', [comp.Owner.Name, comp.Name]);
        defName := comp.ClassName + '1';
        compText[0] :=  StringReplace(
          compText[0],
          Format('object %s:', [comp.Name]),
          Format('object %s:', [defName]),
          []);

        // Save the component text
        componentsText.AddStrings(compText);

        // Save the name to clean up referenes later
        names.Values[curName] := defName;
      end;
    end;

    // Clean up
    for lineIdx := componentsText.Count - 1 downto 0 do
    begin
      // Remove Left & Top
      line := Trim(copy(TrimLeft(componentsText[lineIdx]), 1, 6));
      if (line = 'Left =') or (line = 'Top =') then
        componentsText.Delete(lineIdx)
      else begin
        // Clean up name references
        line := componentsText[lineIdx];
        for nameIdx := 0 to names.Count - 1 do
        begin
          if pos(Format(' = %s',[names.Names[nameIdx]]), line) > 0 then
          begin
            line := line.Replace(names.Names[nameIdx], names.ValueFromIndex[nameIdx], []);
            componentsText[lineIdx] := line;
          end;
        end;
      end;
    end;

    // Put it all in the clipboard
    Clipboard.AsText := componentsText.Text;

  finally
    compText.Free;
    componentsText.Free;
    names.Free;
  end;
end;

function ComponentToText(aComponent: TComponent): string;
var
  StrStream : TStringStream;
  MemStream : TMemoryStream;
begin
  if not Assigned(aComponent) then exit('');

  StrStream := nil;
  MemStream := nil;

  try
    MemStream := TMemoryStream.Create();
    StrStream := TStringStream.Create();
    // Stream the component
    MemStream.WriteComponent(aComponent);
    MemStream.Position := 0;
    // Convert to text
    ObjectBinaryToText(MemStream, StrStream);
    StrStream.Position := 0;
    // Output the text
    Result := StrStream.ReadString(StrStream.Size);
  finally
    StrStream.Free;
    MemStream.Free;
  end;
end;

end.
